@@ -359,14 +359,13 @@ static void hostap_rx_sta_beacon(local_info_t *local, struct sk_buff *skb,
 				 int stype)
 {
 	struct hostap_ieee80211_mgmt *mgmt;
-	int left;
+	int left, chan = 0;
 	u8 *pos;
 	u8 *ssid = NULL, *wpa = NULL, *rsn = NULL;
 	size_t ssid_len = 0, wpa_len = 0, rsn_len = 0;
 	struct hostap_bss_info *bss;
 
-	if (!local->wpa ||
-	    skb->len < IEEE80211_MGMT_HDR_LEN + sizeof(mgmt->u.beacon))
+	if (skb->len < IEEE80211_MGMT_HDR_LEN + sizeof(mgmt->u.beacon))
 		return;
 
 	mgmt = (struct hostap_ieee80211_mgmt *) skb->data;
@@ -393,6 +392,10 @@ static void hostap_rx_sta_beacon(local_info_t *local, struct sk_buff *skb,
 			rsn = pos;
 			rsn_len = pos[1] + 2;
 			break;
+		case WLAN_EID_DS_PARAMS:
+			if (pos[1] >= 1)
+				chan = pos[2];
+			break;
 		}
 		left -= 2 + pos[1];
 		pos += 2 + pos[1];
@@ -423,6 +426,7 @@ static void hostap_rx_sta_beacon(local_info_t *local, struct sk_buff *skb,
 			bss->rsn_ie_len = rsn_len;
 		} else
 			bss->rsn_ie_len = 0;
+		bss->chan = chan;
 	}
 	__hostap_expire_bss(local);
 	spin_unlock(&local->lock);
