@@ -2587,6 +2587,31 @@ static int prism2_hostapd_set_flags_sta(struct ap_data *ap,
 }
 
 
+static int prism2_hostapd_sta_clear_stats(struct ap_data *ap,
+					  struct prism2_hostapd_param *param)
+{
+	struct sta_info *sta;
+	int rate;
+
+	spin_lock_bh(&ap->sta_table_lock);
+	sta = ap_get_sta(ap, param->sta_addr);
+	if (sta) {
+		sta->rx_packets = sta->tx_packets = 0;
+		sta->rx_bytes = sta->tx_bytes = 0;
+		for (rate = 0; rate < WLAN_RATE_COUNT; rate++) {
+			sta->tx_count[rate] = 0;
+			sta->rx_count[rate] = 0;
+		}
+	}
+	spin_unlock_bh(&ap->sta_table_lock);
+
+	if (!sta)
+		return -ENOENT;
+
+	return 0;
+}
+
+
 static int prism2_hostapd(struct ap_data *ap,
 			  struct prism2_hostapd_param *param)
 {
@@ -2602,6 +2627,8 @@ static int prism2_hostapd(struct ap_data *ap,
 		return prism2_hostapd_get_info_sta(ap, param);
 	case PRISM2_HOSTAPD_SET_FLAGS_STA:
 		return prism2_hostapd_set_flags_sta(ap, param);
+	case PRISM2_HOSTAPD_STA_CLEAR_STATS:
+		return prism2_hostapd_sta_clear_stats(ap, param);
 	default:
 		printk(KERN_WARNING "prism2_hostapd: unknown cmd=%d\n",
 		       param->cmd);
