@@ -27,6 +27,15 @@
 #include "hostap_wlan.h"
 #include "hostap_80211.h"
 
+#ifdef CONFIG_CRYPTO
+#ifdef HOSTAP_USE_CRYPTO_API
+#include <linux/crypto.h>
+#include <asm/scatterlist.h>
+#endif /* HOSTAP_USE_CRYPTO_API */
+#else /* CONFIG_CRYPTO */
+#undef HOSTAP_USE_CRYPTO_API
+#endif /* CONFIG_CRYPTO */
+
 MODULE_AUTHOR("Jouni Malinen");
 MODULE_DESCRIPTION("Host AP crypt: CCMP");
 MODULE_LICENSE("GPL");
@@ -43,8 +52,6 @@ struct hostap_ccmp_data {
 	u8 key[CCMP_TK_LEN];
 	int key_set;
 
-	u32 rk[44]; /* AES key data; does not change for each encrypt/decrypt
-		     * operation */
 	u8 tx_pn[CCMP_PN_LEN];
 	u8 rx_pn[CCMP_PN_LEN];
 
@@ -53,9 +60,41 @@ struct hostap_ccmp_data {
 	u32 dot11RSNAStatsCCMPDecryptErrors;
 
 	int key_idx;
+
+#ifdef HOSTAP_USE_CRYPTO_API
+	struct crypto_tfm *tfm;
+#else /* HOSTAP_USE_CRYPTO_API */
+	u32 tfm[44]; /* AES key data; does not change for each encrypt/decrypt
+		      * operation */
+#endif /* HOSTAP_USE_CRYPTO_API */
+
+	/* scratch buffers for virt_to_page() (crypto API) */
+	u8 tx_b0[AES_BLOCK_LEN], tx_b[AES_BLOCK_LEN],
+		tx_e[AES_BLOCK_LEN], tx_s0[AES_BLOCK_LEN];
+	u8 rx_b0[AES_BLOCK_LEN], rx_b[AES_BLOCK_LEN], rx_a[AES_BLOCK_LEN];
 };
 
 
+#ifdef HOSTAP_USE_CRYPTO_API
+
+void hostap_ccmp_aes_encrypt(struct crypto_tfm *tfm,
+			     const u8 pt[16], u8 ct[16])
+{
+	struct scatterlist src, dst;
+
+	src.page = virt_to_page(pt);
+	src.offset = offset_in_page(pt);
+	src.length = AES_BLOCK_LEN;
+
+	dst.page = virt_to_page(ct);
+	dst.offset = offset_in_page(ct);
+	dst.length = AES_BLOCK_LEN;
+
+	crypto_cipher_encrypt(tfm, &dst, &src, AES_BLOCK_LEN);
+}
+
+#else /* HOSTAP_USE_CRYPTO_API */
+
 /* ---- begin public domain AES implementation ----------------------------- */
 /*
  * Modifications to public domain implementation:
@@ -646,6 +685,11 @@ void rijndaelEncrypt(const u32 rk[/*44*/], const u8 pt[16], u8 ct[16])
 }
 /* ---- end public domain AES implementation ------------------------------- */
 
+#define hostap_ccmp_aes_encrypt(t, a, b) rijndaelEncrypt(t, a, b)
+#define crypto_cipher_setkey(t, k, l) rijndaelKeySetupEnc(t, k)
+
+#endif /* HOSTAP_USE_CRYPTO_API */
+
 
 static void * hostap_ccmp_init(int key_idx)
 {
@@ -660,22 +704,46 @@ static void * hostap_ccmp_init(int key_idx)
 
 	priv = (struct hostap_ccmp_data *) kmalloc(sizeof(*priv), GFP_ATOMIC);
 	if (priv == NULL) {
-#ifdef NEW_MODULE_CODE
-		module_put(THIS_MODULE);
-#else
-		MOD_DEC_USE_COUNT;
-#endif
-		return NULL;
+		goto fail;
 	}
 	memset(priv, 0, sizeof(*priv));
 	priv->key_idx = key_idx;
 
+#ifdef HOSTAP_USE_CRYPTO_API
+	priv->tfm = crypto_alloc_tfm("aes", 0);
+	if (priv->tfm == NULL) {
+		printk(KERN_DEBUG "hostap_crypt_ccmp: could not allocate "
+		       "crypto API aes\n");
+		goto fail;
+	}
+#endif /* HOSTAP_USE_CRYPTO_API */
+
 	return priv;
+
+fail:
+	if (priv) {
+#ifdef HOSTAP_USE_CRYPTO_API
+		if (priv->tfm)
+			crypto_free_tfm(priv->tfm);
+#endif /* HOSTAP_USE_CRYPTO_API */
+		kfree(priv);
+	}
+#ifdef NEW_MODULE_CODE
+	module_put(THIS_MODULE);
+#else
+	MOD_DEC_USE_COUNT;
+#endif
+	return NULL;
 }
 
 
 static void hostap_ccmp_deinit(void *priv)
 {
+#ifdef HOSTAP_USE_CRYPTO_API
+	struct hostap_ccmp_data *_priv = priv;
+	if (_priv && _priv->tfm)
+		crypto_free_tfm(_priv->tfm);
+#endif /* HOSTAP_USE_CRYPTO_API */
 	kfree(priv);
 #ifdef NEW_MODULE_CODE
 	module_put(THIS_MODULE);
@@ -693,14 +761,20 @@ static inline void xor_block(u8 *b, u8 *a, size_t len)
 }
 
 
-static void ccmp_init_blocks(u32 rk[], struct hostap_ieee80211_hdr *hdr,
-			     u8 *pn, size_t dlen, u8 *b0, u8 *aad, u8 *auth,
+#ifdef HOSTAP_USE_CRYPTO_API
+static void ccmp_init_blocks(struct crypto_tfm *tfm,
+#else /* HOSTAP_USE_CRYPTO_API */
+static void ccmp_init_blocks(u32 tfm[],
+#endif /* HOSTAP_USE_CRYPTO_API */
+			     struct hostap_ieee80211_hdr *hdr,
+			     u8 *pn, size_t dlen, u8 *b0, u8 *auth,
 			     u8 *s0)
 {
 	u8 *pos, qc = 0;
 	size_t aad_len;
 	u16 fc;
 	int a4_included, qc_included;
+	u8 aad[2 * AES_BLOCK_LEN];
 
 	fc = le16_to_cpu(hdr->frame_control);
 	a4_included = ((fc & (WLAN_FC_TODS | WLAN_FC_FROMDS)) ==
@@ -755,14 +829,14 @@ static void ccmp_init_blocks(u32 rk[], struct hostap_ieee80211_hdr *hdr,
 	}
 
 	/* Start with the first block and AAD */
-	rijndaelEncrypt(rk, b0, auth);
+	hostap_ccmp_aes_encrypt(tfm, b0, auth);
 	xor_block(auth, aad, AES_BLOCK_LEN);
-	rijndaelEncrypt(rk, auth, auth);
+	hostap_ccmp_aes_encrypt(tfm, auth, auth);
 	xor_block(auth, &aad[AES_BLOCK_LEN], AES_BLOCK_LEN);
-	rijndaelEncrypt(rk, auth, auth);
+	hostap_ccmp_aes_encrypt(tfm, auth, auth);
 	b0[0] &= 0x07;
 	b0[14] = b0[15] = 0;
-	rijndaelEncrypt(rk, b0, s0);
+	hostap_ccmp_aes_encrypt(tfm, b0, s0);
 }
 
 
@@ -772,8 +846,10 @@ static int hostap_ccmp_encrypt(struct sk_buff *skb, int hdr_len, void *priv)
 	int data_len, i, blocks, last, len;
 	u8 *pos, *mic;
 	struct hostap_ieee80211_hdr *hdr;
-	u8 aad[2 * AES_BLOCK_LEN], b0[AES_BLOCK_LEN], b[AES_BLOCK_LEN],
-		e[AES_BLOCK_LEN], s0[AES_BLOCK_LEN];
+	u8 *b0 = key->tx_b0;
+	u8 *b = key->tx_b;
+	u8 *e = key->tx_e;
+	u8 *s0 = key->tx_s0;
 
 	if (skb_headroom(skb) < CCMP_HDR_LEN ||
 	    skb_tailroom(skb) < CCMP_MIC_LEN ||
@@ -804,7 +880,7 @@ static int hostap_ccmp_encrypt(struct sk_buff *skb, int hdr_len, void *priv)
 	*pos++ = key->tx_pn[0];
 
 	hdr = (struct hostap_ieee80211_hdr *) skb->data;
-	ccmp_init_blocks(key->rk, hdr, key->tx_pn, data_len, b0, aad, b, s0);
+	ccmp_init_blocks(key->tfm, hdr, key->tx_pn, data_len, b0, b, s0);
 
 	blocks = (data_len + AES_BLOCK_LEN - 1) / AES_BLOCK_LEN;
 	last = data_len % AES_BLOCK_LEN;
@@ -813,11 +889,11 @@ static int hostap_ccmp_encrypt(struct sk_buff *skb, int hdr_len, void *priv)
 		len = (i == blocks && last) ? last : AES_BLOCK_LEN;
 		/* Authentication */
 		xor_block(b, pos, len);
-		rijndaelEncrypt(key->rk, b, b);
+		hostap_ccmp_aes_encrypt(key->tfm, b, b);
 		/* Encryption, with counter */
 		b0[14] = (i >> 8) & 0xff;
 		b0[15] = i & 0xff;
-		rijndaelEncrypt(key->rk, b0, e);
+		hostap_ccmp_aes_encrypt(key->tfm, b0, e);
 		xor_block(pos, e, len);
 		pos += len;
 	}
@@ -834,8 +910,9 @@ static int hostap_ccmp_decrypt(struct sk_buff *skb, int hdr_len, void *priv)
 	struct hostap_ccmp_data *key = priv;
 	u8 keyidx, *pos;
 	struct hostap_ieee80211_hdr *hdr;
-	u8 aad[2 * AES_BLOCK_LEN], b0[AES_BLOCK_LEN], b[AES_BLOCK_LEN],
-		a[AES_BLOCK_LEN];
+	u8 *b0 = key->rx_b0;
+	u8 *b = key->rx_b;
+	u8 *a = key->rx_a;
 	u8 pn[6];
 	int i, blocks, last, len;
 	size_t data_len = skb->len - hdr_len - CCMP_HDR_LEN - CCMP_MIC_LEN;
@@ -892,7 +969,7 @@ static int hostap_ccmp_decrypt(struct sk_buff *skb, int hdr_len, void *priv)
 		return -4;
 	}
 
-	ccmp_init_blocks(key->rk, hdr, pn, data_len, b0, aad, a, b);
+	ccmp_init_blocks(key->tfm, hdr, pn, data_len, b0, a, b);
 	xor_block(mic, b, CCMP_MIC_LEN);
 
 	blocks = (data_len + AES_BLOCK_LEN - 1) / AES_BLOCK_LEN;
@@ -903,11 +980,11 @@ static int hostap_ccmp_decrypt(struct sk_buff *skb, int hdr_len, void *priv)
 		/* Decrypt, with counter */
 		b0[14] = (i >> 8) & 0xff;
 		b0[15] = i & 0xff;
-		rijndaelEncrypt(key->rk, b0, b);
+		hostap_ccmp_aes_encrypt(key->tfm, b0, b);
 		xor_block(pos, b, len);
 		/* Authentication */
 		xor_block(a, pos, len);
-		rijndaelEncrypt(key->rk, a, a);
+		hostap_ccmp_aes_encrypt(key->tfm, a, a);
 		pos += len;
 	}
 
@@ -935,10 +1012,16 @@ static int hostap_ccmp_set_key(void *key, int len, u8 *seq, void *priv)
 {
 	struct hostap_ccmp_data *data = priv;
 	int keyidx;
+#ifdef HOSTAP_USE_CRYPTO_API
+	struct crypto_tfm *tfm = data->tfm;
+#endif /* HOSTAP_USE_CRYPTO_API */
 
 	keyidx = data->key_idx;
 	memset(data, 0, sizeof(*data));
 	data->key_idx = keyidx;
+#ifdef HOSTAP_USE_CRYPTO_API
+	data->tfm = tfm;
+#endif /* HOSTAP_USE_CRYPTO_API */
 	if (len == CCMP_TK_LEN) {
 		memcpy(data->key, key, CCMP_TK_LEN);
 		data->key_set = 1;
@@ -950,7 +1033,7 @@ static int hostap_ccmp_set_key(void *key, int len, u8 *seq, void *priv)
 			data->rx_pn[4] = seq[1];
 			data->rx_pn[5] = seq[0];
 		}
-		rijndaelKeySetupEnc(data->rk, data->key);
+		crypto_cipher_setkey(data->tfm, data->key, CCMP_TK_LEN);
 	} else if (len == 0) {
 		data->key_set = 0;
 	} else
