@@ -348,7 +348,7 @@ static int prism2_pci_suspend(struct pci_dev *pdev, u32 state)
 		netif_stop_queue(dev);
 		netif_device_detach(dev);
 	}
-	prism2_hw_shutdown(dev, 0);
+	prism2_suspend(dev);
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10))
 	pci_save_state(pdev, ((struct hostap_interface *) dev->priv)->
 		       local->pci_save_state);
